'''
This script creates the endpoints for the API.

@ Author: Paul Bethge
@ Date: 01/09/2022

Copyright (c) 2022 ZKM | Karsruhe
Simplified BSD License
'''

# Imports
import os
import sys
import json
import base64
import logging
import logging.config as cnfg
import time
import torch
from typing import Union
from pydantic import BaseModel
from detoxify import Detoxify

from fastapi import FastAPI, HTTPException
from fastapi.middleware.cors import CORSMiddleware


# ==== Configuration ==== #

# configure logging
logging.config.fileConfig('log.ini', disable_existing_loggers=False)
logger = logging.getLogger(__name__)  # the __name__ resolve to "main" since we are at the root of the project. 
									# This will get the root logger since no logger in the configuration has this name.

device = 'cuda' if torch.cuda.is_available() else 'cpu'
logger.info('Using device: {}'.format(device))

# ==== Setup ==== #
app = FastAPI()

# ===  CORS === #
origins = ["*"]
app.add_middleware(
	CORSMiddleware,
	allow_origins=origins,
	allow_credentials=True,
	allow_methods=["*"],
	allow_headers=["*"],
)

# === Request Models === #
class BasicRequest(BaseModel):
	sentences: list


# === ML Models === #
model = Detoxify('unbiased')


async def predict(sentences: list):
    try:
        # make prediction
        results = model.predict(sentences)
        # return results
        return results
    except Exception as e:
        logger.error(e)
        raise HTTPException(status_code=500, detail="Internal Server Error")


# === Endpoints === #
@app.post("/detoxify")
async def detoxify(request: BasicRequest):
    # make prediction
    results = await predict(request.sentences)
    # return results
    return results